##############################################################################
#
# Copyright (c) 2010 ViFiB SARL and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


import os
import posixpath
import hashlib
from libnetworkcache import NetworkcacheClient


def _get_hash_from_file(path, hash_object):
    """
      Calculate the hash from file.
    """
    f = open(path, 'rb')
    try:
        chunk = f.read(2**16)
        while chunk:
            hash_object.update(chunk)
            chunk = f.read(2**16)
        return hash_object.hexdigest()
    finally:
        f.close()


def check_sha256sum(path, sha256sum):
    """Tell whether the SHA256sum checksum of the file at path matches
    """
    return sha256sum == _get_hash_from_file(path, hashlib.sha256())


def _update_network_cached_log(key, url):
    """Update the networkcached.log file.

    Adding the last key and URL which has been retrieved from networkcached server.

    """
    # Keep the log
    map_path = os.path.join(os.environ.get('PWD'), '.networkcached.log')
    mode = 'r+'
    if not os.path.exists(map_path):
        mode = 'w+'
    f = open(map_path, mode)
    try:
        flag = False
        for line in f:
            if url in line:
                flag = True
                break
        if not flag:
          f.seek(0, 2)
          f.write('%s %s\n' % (key, url))
          f.truncate(f.tell())
    finally:
        f.close()


def download_network_cached(network_cache, path, url, logger):
    """Download from a network cache provider

    If something fail (providor be offline, or hash_string fail), we ignore
    network cached files.

    return True if download succeeded.
    """
    if network_cache in (None, '',):
        # Not able to use network cache
        return False

    key = hashlib.sha256(url).hexdigest()
    network_cached_url = os.path.join(network_cache, key)
    logger.info('Downloading from network cache %s' % network_cached_url)
    try:
        nc = NetworkcacheClient(network_cache)
        result, path = nc.get(key, path)
        if result.status != 200:
          logger.info('Fail to download from network cache: File Not Found.')
          return False

        if not check_sha256sum(path, result.getheader('x-sha256sum')):
            logger.info('SHA256 checksum mismatch downloading %r' % \
                                                           network_cached_url)
            return False
    except IOError, e:
        logger.info('Fail to download from network cache %s: %s' % \
                                                  (network_cached_url, str(e)))
        return False

    return True


def upload_network_cached(network_cache, external_url, path, logger):
    """Upload file to a network cache server"""
    if network_cache in [None, '']:
        logger.debug(
         'Upload cache ignored, network-cache was not provided')
        return False

    key = hashlib.sha256(external_url).hexdigest()
    try:
        nc = NetworkcacheClient(network_cache)
        result = nc.put(key, path)
        if result.status == 200:
            _update_network_cached_log(key, external_url)

    except (IOError, EOFError), e:
        logger.info('Fail to upload cache on %s. %s' % \
                                                  (external_url, str(e)))
    return True


def get_filename_from_url(url):
    """Inspired how pip get filename from url.
    """
    url = url.split('#', 1)[0]
    url = url.split('?', 1)[0]
    url = url.rstrip('/')
    name = posixpath.basename(url)
    assert name, (
           'URL %r produced no filename' % url)
    return name
